<?php

namespace Drupal\memcache_storage;

use Drupal\Core\Site\Settings;
use Psr\Log\LogLevel;

class DrupalMemcachedInitializeException extends \Exception {};

class DrupalMemcachedFactory {

  protected $settings;

  protected $extension;

  protected $servers;

  protected $bins_clusters;

  protected $connections;

  public function __construct(Settings $settings) {
    $this->extension = DrupalMemcachedUtils::getPeclExtension();
    if (!class_exists($this->extension) || !in_array($this->extension, ['Memcache', 'Memcached'])) {
      register_shutdown_function('DrupalMemcachedUtils::log', LogLevel::ERROR, 'Could not initialize PECL extension. Make sure that it exists on the server.');
      throw new DrupalMemcachedInitializeException('Could not initialize PECL extension.');
    }

    $this->settings = $settings;
    $this->servers = $settings->get('memcache_storage.memcached_servers', ['127.0.0.1:11211' => 'default']);
    $this->bins_clusters = $settings->get('memcache_storage.bins_clusters', []);
  }

  public function get($bin) {

    // Get the name of the server cluster for the selected cache bin.
    $cluster_name = !empty($this->bins_clusters[$bin]) ? $this->bins_clusters[$bin] : 'default';

    if (!isset($this->connections[$cluster_name])) {

      // Search for all servers matching bin cluster.
      foreach ($this->servers as $server => $name) {
        if ($name == $cluster_name) {
          $cluster_servers[] = $server;
        }
      }

      // No servers found for this cache bin.
      if (empty($cluster_servers)) {
        DrupalMemcachedUtils::log(LogLevel::ERROR, 'Could not find server for cluster %cluster.', ['%cluster' => $cluster_name]);
        return FALSE;
      }

      // TODO: Switch to services.
      $class_name = 'Drupal\memcache_storage\Drupal' . ucfirst(strtolower($this->extension));
      $memcached = new $class_name($this->settings, $cluster_servers, $cluster_name);
      if (!$memcached->isConnected()) {
        return FALSE;
      }

      // Store memcached object as a static object, to avoid duplicate connections to the same
      // memcached cluster but for another cache bin.
      $this->connections[$cluster_name] = $memcached;
    }

    return $this->connections[$cluster_name];
  }

}
