<?php

namespace Drupal\memcache_storage;

use Drupal\Core\Site\Settings;

class DrupalMemcachedUtils {

  /**
   * Parses the string of memcached server.
   *
   * @param $memcached_server string
   *   Examples of server string:
   *   - unix:///path/to/memcached.socket1
   *   - 127.0.0.1:12111
   *
   * @return array
   *   Processed host and port values.
   */
  public static function parseServerInfo($memcached_server) {
    list($host, $port) = explode(':', $memcached_server);

    // Support unix sockets in the format 'unix:///path/to/socket'.
    if ($host == 'unix') {

      // PECL Memcache requires string format like 'unix:///path/to/socket' to
      // establish a connection, while PECL Memcached requires only
      // '/path/to/socket' string for the same purpose.
      $pecl_extension = self::getPeclExtension();
      $host = $pecl_extension == 'Memcached' ? substr($memcached_server, 7) : $memcached_server;

      // For unix sockets port is always 0.
      $port = 0;
    }

    return [$host, $port];
  }

  /**
   * Returns the name of PECL extension to use.
   */
  public static function getPeclExtension() {
    $preferred = Settings::get('memcache_storage.extension');
    if (!empty($preferred)) {
      $extension = $preferred;
    }

    // If no extension is set, default to PECL Memcached.
    elseif (class_exists('Memcached')) {
      $extension = 'Memcached';
    }
    elseif (class_exists('Memcache')) {
      $extension = 'Memcache';
    }

    return !empty($extension) ? $extension : FALSE;
  }

  /**
   * Logs a message.
   *
   * @param $level
   * @param $message
   * @param array $context
   */
  public static function log($level, $message, array $context = []) {
    \Drupal::logger('memcache_storage')->log($level, $message, $context);
  }

}
